@@ -130,6 +130,40 @@ function needHomepageOverride(prefb) {
   return 0;
 }
 
+// Copies a pref override file into the user's profile pref-override folder,
+// and then tells the pref service to reload it's default prefs.
+function copyPrefOverride() {
+  try {
+    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
+                                .getService(Components.interfaces.nsIProperties);
+    const NS_APP_EXISTING_PREF_OVERRIDE = "ExistingPrefOverride";
+    var prefOverride = fileLocator.get(NS_APP_EXISTING_PREF_OVERRIDE,
+                                       Components.interfaces.nsIFile);
+    if (!prefOverride.exists())
+      return; // nothing to do
+
+    const NS_APP_PREFS_OVERRIDE_DIR     = "PrefDOverride";
+    var prefOverridesDir = fileLocator.get(NS_APP_PREFS_OVERRIDE_DIR,
+                                           Components.interfaces.nsIFile);
+
+    // Check for any existing pref overrides, and remove them if present
+    var existingPrefOverridesFile = prefOverridesDir.clone();
+    existingPrefOverridesFile.append(prefOverride.leafName);
+    if (existingPrefOverridesFile.exists())
+      existingPrefOverridesFile.remove(false);
+
+    prefOverride.copyTo(prefOverridesDir, null);
+
+    // Now that we've installed the new-profile pref override file,
+    // re-read the default prefs.
+    var prefSvcObs = Components.classes["@mozilla.org/preferences-service;1"]
+                               .getService(Components.interfaces.nsIObserver);
+    prefSvcObs.observe(null, "reload-default-prefs", null);
+  } catch (ex) {
+    Components.utils.reportError(ex);
+  }
+}
+
 function openWindow(parent, url, target, features, args) {
   var wwatch = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                          .getService(nsIWindowWatcher);
@@ -161,14 +195,14 @@ function getMostRecentWindow(aType) {
   return wm.getMostRecentWindow(aType);
 }
 
-//@line 169 "/export/home/mozilla/uild/firefox2.0/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 203 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
 
 // this returns the most recent non-popup browser window
 function getMostRecentBrowserWindow() {
   var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                      .getService(Components.interfaces.nsIWindowMediator);
 
-//@line 176 "/export/home/mozilla/uild/firefox2.0/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 210 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
   var win = wm.getMostRecentWindow("navigator:browser", true);
 
   // if we're lucky, this isn't a popup, and we can just return this
@@ -181,7 +215,7 @@ function getMostRecentBrowserWindow() {
         win = nextWin;
     }
   }
-//@line 201 "/export/home/mozilla/uild/firefox2.0/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 235 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
 
   return win;
 }
@@ -266,14 +300,17 @@ var nsBrowserContentHandler = {
     if (remoteCommand != null) {
       try {
         var a = /^\s*(\w+)\(([^\)]*)\)\s*$/.exec(remoteCommand);
-        var remoteVerb = a[1].toLowerCase();
-        var remoteParams = [];
-        var sepIndex = a[2].lastIndexOf(",");
-        if (sepIndex == -1)
-          remoteParams[0] = a[2];
-        else {
-          remoteParams[0] = a[2].substring(0, sepIndex);
-          remoteParams[1] = a[2].substring(sepIndex + 1);
+        var remoteVerb;
+        if (a) {
+          remoteVerb = a[1].toLowerCase();
+          var remoteParams = [];
+          var sepIndex = a[2].lastIndexOf(",");
+          if (sepIndex == -1)
+            remoteParams[0] = a[2];
+          else {
+            remoteParams[0] = a[2].substring(0, sepIndex);
+            remoteParams[1] = a[2].substring(sepIndex + 1);
+          }
         }
 
         switch (remoteVerb) {
@@ -283,15 +320,26 @@ var nsBrowserContentHandler = {
           // openURL(<url>,new-window)
           // openURL(<url>,new-tab)
 
-          var uri = resolveURIInternal(cmdLine, remoteParams[0]);
+          // First param is the URL, second param (if present) is the "target"
+          // (tab, window)
+          var url = remoteParams[0];
+          var target = nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW;
+          if (remoteParams[1]) {
+            var targetParam = remoteParams[1].toLowerCase()
+                                             .replace(/^\s*|\s*$/g, "");
+            if (targetParam == "new-tab")
+              target = nsIBrowserDOMWindow.OPEN_NEWTAB;
+            else if (targetParam == "new-window")
+              target = nsIBrowserDOMWindow.OPEN_NEWWINDOW;
+            else {
+              // The "target" param isn't one of our supported values, so
+              // assume it's part of a URL that contains commas.
+              url += "," + remoteParams[1];
+            }
+          }
 
-          var location = nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW;
-          if (/new-window/.test(remoteParams[1]))
-            location = nsIBrowserDOMWindow.OPEN_NEWWINDOW;
-          else if (/new-tab/.test(remoteParams[1]))
-            location = nsIBrowserDOMWindow.OPEN_NEWTAB;
-
-          handURIToExistingBrowser(uri, location);
+          var uri = resolveURIInternal(cmdLine, url);
+          handURIToExistingBrowser(uri, target, cmdLine);
           break;
 
         case "xfedocommand":
@@ -307,12 +355,13 @@ var nsBrowserContentHandler = {
         default:
           // Somebody sent us a remote command we don't know how to process:
           // just abort.
-          throw NS_ERROR_ABORT;
+          throw "Unknown remote command.";
         }
 
         cmdLine.preventDefault = true;
       }
       catch (e) {
+        Components.utils.reportError(e);
         // If we had a -remote flag but failed to process it, throw
         // NS_ERROR_ABORT so that the xremote code knows to return a failure
         // back to the handling code.
@@ -339,7 +388,7 @@ var nsBrowserContentHandler = {
     try {
       while ((uriparam = cmdLine.handleFlagWithParam("new-tab", false))) {
         var uri = resolveURIInternal(cmdLine, uriparam);
-        handURIToExistingBrowser(uri, nsIBrowserDOMWindow.OPEN_NEWTAB);
+        handURIToExistingBrowser(uri, nsIBrowserDOMWindow.OPEN_NEWTAB, cmdLine);
         cmdLine.preventDefault = true;
       }
     }
@@ -373,7 +422,7 @@ var nsBrowserContentHandler = {
       cmdLine.preventDefault = true;
     }
 
-//@line 405 "/export/home/mozilla/uild/firefox2.0/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 454 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
   },
 
   helpInfo : "  -browser            Open a browser window.\n",
@@ -390,9 +439,13 @@ var nsBrowserContentHandler = {
     var overrideState = needHomepageOverride(prefb);
     try {
       if (overrideState == 1) {
+        // New profile.
         pagesToLoad = formatter.formatURLPref("startup.homepage_welcome_url");
       }
       else if (overrideState == 2) {
+        // Existing profile.
+        copyPrefOverride();
+
         pagesToLoad = formatter.formatURLPref("startup.homepage_override_url");
       }
     }
@@ -520,7 +573,7 @@ const bch_contractID = "@mozilla.org/browser/clh;1";
 const bch_CID = Components.ID("{5d0ce354-df01-421a-83fb-7ead0990c24e}");
 const CONTRACTID_PREFIX = "@mozilla.org/uriloader/content-handler;1?type=";
 
-function handURIToExistingBrowser(uri, location)
+function handURIToExistingBrowser(uri, location, cmdLine)
 {
   if (!shouldLoadURI(uri))
     return;
@@ -556,23 +609,49 @@ var nsDefaultCommandLineHandler = {
     return this;
   },
 
+  // List of uri's that were passed via the command line without the app
+  // running and have already been handled. This is compared against uri's
+  // opened using DDE on Win32 so we only open one of the requests.
+  _handledURIs: [ ],
+//@line 647 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
+
   /* nsICommandLineHandler */
   handle : function dch_handle(cmdLine) {
     var urilist = [];
 
+//@line 673 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
+
     try {
       var ar;
       while ((ar = cmdLine.handleFlagWithParam("url", false))) {
-        urilist.push(resolveURIInternal(cmdLine, ar));
+        var found = false;
+        var uri = resolveURIInternal(cmdLine, ar);
+        // count will never be greater than zero except on Win32.
+        var count = this._handledURIs.length;
+        for (var i = 0; i < count; ++i) {
+          if (this._handledURIs[i].spec == uri.spec) {
+            this._handledURIs.splice(i, 1);
+            found = true;
+            cmdLine.preventDefault = true;
+            break;
+          }
+        }
+        if (!found) {
+          urilist.push(uri);
+          // The requestpending command line flag is only used on Win32.
+          if (cmdLine.handleFlag("requestpending", false) &&
+              cmdLine.state == nsICommandLine.STATE_INITIAL_LAUNCH)
+            this._handledURIs.push(uri)
+        }
       }
     }
     catch (e) {
       Components.utils.reportError(e);
     }
 
-    var count = cmdLine.length;
+    count = cmdLine.length;
 
-    for (var i = 0; i < count; ++i) {
+    for (i = 0; i < count; ++i) {
       var curarg = cmdLine.getArgument(i);
       if (curarg.match(/^-/)) {
         Components.utils.reportError("Warning: unrecognized command line flag " + curarg + "\n");
@@ -595,7 +674,7 @@ var nsDefaultCommandLineHandler = {
         // Try to find an existing window and load our URI into the
         // current tab, new tab, or new window as prefs determine.
         try {
-          handURIToExistingBrowser(urilist[0], nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW);
+          handURIToExistingBrowser(urilist[0], nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW, cmdLine);
           return;
         }
         catch (e) {
@@ -603,7 +682,7 @@ var nsDefaultCommandLineHandler = {
       }
 
       var speclist = [];
-      for (var uri in urilist) {
+      for (uri in urilist) {
         if (shouldLoadURI(urilist[uri]))
           speclist.push(urilist[uri].spec);
       }
@@ -690,7 +769,7 @@ var Module = {
 
     registerType("text/html");
     registerType("application/vnd.mozilla.xul+xml");
-//@line 724 "/export/home/mozilla/uild/firefox2.0/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 825 "/export/home/mozilla/uild/firefox-2.0.0.3/src/mozilla/browser/components/nsBrowserContentHandler.js"
     registerType("text/rdf");
     registerType("text/xml");
     registerType("application/xhtml+xml");
